﻿* Encoding: UTF-8.
*Analytic Hierarchy Process
*SPSS macros by Kirill Orlov
*kior@akado.ru, ttnphns@gmail.com
*https://www.spsstools.net/en/KO-spssmacros
*All rights reserved.

*/*!KO_AHP*/vers1***************************************************************************************.
define !KO_ahp(levels= !charend('/') /altlab= !charend('/') /altpri= !charend('/') /print= !charend('/'))
exec.
!if (!altlab<>!null !and !altpri=!null !or !altlab=!null !and !altpri<>!null) !then
-echo ''.
-echo '>ERROR. Subcommands ALTPRI and ALTLAB can be specified only both.'.
-echo '>!KO_AHP is not executed.'.
!else
 !let !levs= !null
 !do !w !in (!levels) !let !levs= !conc(!levs,',',!w) !doend
-set mxloops 1E6.
-sort cases by level (d).
-matrix.
-print /title '**** ANALYTIC HIERARCHY PROCESS ****'.
-get lev /var= level.
-get ovl /var= overlier.
-get node1 /var= node1.
-get node2 /var= node2.
-get win /var= winner.
-get sco /var= score.
-comp levs= {0 !levs}.
 !if (!altpri=!null) !then
- comp cpriormx= 1.
 !else !let !altlbs= !null
  !do !w !in (!altlab) !let !altlbs= !conc(!altlbs,';',!w) !doend
- comp altlab= {!tail(!altlbs)}.
- comp cpriormx= !altpri.
- comp cpriormx= cpriormx/csum(cpriormx)(make(1,nrow(cpriormx),1),:).
- print /title 'Hierarchy will consist of the number of levels one more than declared in'.
- print /title 'LEVELS, because ALTPRI/ALTLAB are specified.' /space= 0.
- print /title 'Input by ALTPRI priority vectors (columns) of alternatives'.
- print lev(1) /title 'relative criteria of the overlying level' /space= 0.
- print cpriormx /title ' ' /rnames= altlab /format= f7.4 /space= 0.
- print /title '---------------'.
 !ifend
-comp lev= {lev;999}.
-comp ovl= {ovl;'999.999'}.
-comp node1= {node1;'999.999'}.
-comp st= 1.
-comp cntovl= 1.
-comp cons= {0;0;0}.
-comp ri= {0,0.01,0.52,0.89,1.12,1.26,1.36,1.41,1.46,1.49,1.52,1.54}.
-comp cwci= 0.
-comp cwri= 0.
-loop en= 1 to nrow(lev)-1.
- do if ovl(en+1)<>ovl(en) or lev(en+1)<>lev(en).
-  comp n= (1+sqrt(1+8*(en-st+1)))/2.
-  do if n>12.
-   print lev(en) /title '>ERROR. The number of nodes subordinate to a criterion is >12 on level:'.
-   print /title '>!KO_AHP is executed incorrectly.' /space= 0.
-   break.
-  end if.
-  comp a= make(n,n,1).
-  comp lab= make(n,1,' ').
-  comp lab(1)= node1(st).
-  comp r= 1.
-  comp c= 2.
-  loop i= st to en.
-   do if r=1.
-    comp lab(c)= node2(i).
-   end if.
-   do if win(i)=node1(i).
-    comp a(r,c)= sco(i).
-   else.
-    comp a(c,r)= sco(i).
-   end if.
-   comp c= c+1.
-   do if node1(i)<>node1(i+1).
-    comp r= r+1.
-    comp c= r+1.
-   end if.
-  end loop.
-  comp a= a/t(a).
   !if (!index(!print,'MATR')>0) !then
-   print lev(en) /title 'Matrix of pairwise comparisons (judgements) on level'.
-   do if lev(en)<>1.
-    print ovl(en) /title 'relative overlying criterion' /format= a8 /space= 0.
-   end if.
-   print a /title= ' ' /rnames= lab /cnames= lab /format= f7.4 /space= 0.
   !ifend
-  comp inivec= exp(rsum(ln(a))/ncol(a)).
-  loop @i= 1 to 100.
-   comp prior= a*inivec.
-   comp prior= prior /sqrt(mssq(prior)).
-   comp @ch= mmax(abs(prior-inivec)).
-   do if @ch<1E-6.
-    break.
-   end if.
-   comp inivec= prior.
-  end loop.
-  comp eival= t(prior)*a*prior.
-  comp prior= prior/msum(prior).
-  comp cons(1)= eival.
-  comp cons(2)= (eival-n)/(n-1).
-  comp cons(3)= cons(2)/ri(n).
   !if (!index(!print,'CONSIST')>0) !then
    !if (!index(!print,'MATR')=0) !then
-    print lev(en) /title 'Consistency of pairwise comparisons on level'.
-    do if lev(en)<>1.
-     print ovl(en) /title 'relative overlying criterion' /format= a8 /space= 0.
-    end if.
-   !else
-    print /title 'and consistency of pairwise comparisons:'.
    !ifend
-   print cons /title= ' ' /rlab= 'Lmax' 'CI' 'CR' /format= f7.4 /space= 0.
   !ifend
-  do if cntovl=1.
-   comp priormx= prior.
-   comp rlab= lab.
-   comp clab= ovl(en).
-   comp civec= cons(2).
-   comp rivec= ri(n).
-  else.
-   do if n=levs(ncol(levs)).
-    comp priormx= {priormx,prior}.
-   else.
-    comp priormx= block(priormx,prior).
-    comp rlab= {rlab;lab}.
-   end if.
-   comp clab= {clab,ovl(en)}.
-   comp civec= {civec,cons(2)}.
-   comp rivec= {rivec,ri(n)}.
-  end if.
-  comp st= en+1.
-  comp cntovl= cntovl+1.
- end if.
- do if lev(en+1)<>lev(en).
   !if (!index(!print,'PRIOR')>0) !then
-   print lev(en) /title 'Priority vectors (columns) on level'.
-   do if lev(en)=1.
-    print priormx /title 'relative the Goal:' /rnames= rlab /cnames= clab /format= f7.4 /space= 0.
-   else.
-    print priormx /title 'relative criteria of the overlying level:' /rnames= rlab /cnames= clab /format= f7.4 /space= 0.
-   end if.
   !ifend
-  comp cpriormx= cpriormx*priormx.
-  do if lev(en)<>lev(1).
-   comp cwci= cwci+civec_*rsum(priormx).
-   comp cwri= cwri+rivec_*rsum(priormx).
-  end if.
-  comp civec_= civec.
-  comp rivec_= rivec.
   !if (!altlab=!null) !then
-   do if lev(en)=lev(1).
-    comp altlab= rlab.
-   end if.
   !ifend
-  do if lev(en)=1.
-   print cpriormx /title 'Priority vector of alternatives relative the Goal:' /format= f7.4 /rnames= altlab.
-  end if.
-  comp levs= levs(1:(ncol(levs)-1)).
-  comp cntovl= 1.
   !if (!print<>!null) !then
-   print /title '---------------'.
   !ifend
- end if.
-end loop.
-print ((cwci+cons(2))/(cwri+ri(n)))
    /title !if (!altpri=!null) !then 'Consistency ratio of hierarchy:' !else 'Consistency ratio of hierarchy (without the bottom level):' !ifend
    /rlab= 'CRH' /format= f7.4.
-end matrix.
!ifend
!enddefine.
